/* Copyright 2009 Marcel Overdijk
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.grails.samples.sakila

class AuthController {

    static allowedMethods = [login: "POST", updatePassword: "POST"]

    def index = {
        if (session.user) {
            redirect(uri: "/")
        }
        else {
            render(view: "login")
        }
    }

    def login = { LoginCommand cmd ->
        if (!cmd.hasErrors()) {
            def staffInstance = Staff.findByUsernameIlike(cmd.username)
            if (staffInstance && staffInstance.active && staffInstance.password == cmd.password.encodeAsSHA1Hex()) {
                session.user = staffInstance
            }
            else {
                cmd.errors.reject("auth.login.invalid", "Username or password is not valid")
            }
        }
        if (cmd.hasErrors()) {
            render(view: "login", model: [loginCommand: cmd])
        }
        else {
            redirect(uri: "/")
        }
    }

    def logout = {
        session.user = null
        session.invalidate()
        flash.message = "auth.logged.out"
        flash.defaultMessage = "You have successfully logged out"
        redirect(action: "index")
    }

    def changePassword = { ChangePasswordCommand cmd ->
        def staffInstance = Staff.get(session.user.id)
        if (!staffInstance) {
            redirect(uri: "/")
        }
        else {
            return [changePasswordCommand: cmd]
        }
    }

    def updatePassword = { ChangePasswordCommand cmd ->
        if (!cmd.hasErrors()) {
            def staffInstance = Staff.get(session.user.id)
            if (staffInstance && staffInstance.password == cmd.oldPassword.encodeAsSHA1Hex()) {
                staffInstance.password = cmd.newPassword.encodeAsSHA1Hex()
                if (staffInstance.hasErrors() || !staffInstance.save()) {
                    cmd.errors.reject("auth.change.password.error", "Password not changed")
                }
            }
            else {
                cmd.errors.reject("auth.change.password.oldPassword.invalid", "Old Password is not valid")
            }
        }
        if (cmd.hasErrors()) {
            render(view: "changePassword", model: [changePasswordCommand: cmd])
        }
        else {
            flash.message = "auth.password.changed"
            flash.defaultMessage = "You have successfully changed your password"
            redirect(uri: "/")
        }
    }
}

class LoginCommand {

    String username
    String password

    static constraints = {
        username(blank: false)
        password(blank: false)
    }
}

class ChangePasswordCommand {
    
    String oldPassword
    String newPassword
    String confirmNewPassword

    static constraints = {
        oldPassword(blank: false)
        newPassword(blank: false)
        confirmNewPassword(blank: false, validator: { val, obj -> obj.properties["newPassword"] == val })
    }
}

