/* Copyright 2009 Marcel Overdijk
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.grails.samples.sakila

class CategoryController extends BaseController {

    def index = { redirect(action: "list", params: params) }
    
    static allowedMethods = [save: "POST", update: "POST", delete: "POST", search: "POST"]

    def list = {
        if (params.max) params.max = Math.min(params.max.toInteger(), 100)
        restoreState([max: 10, offset: 0, sort: "id", order: "asc", search: ""])
        def categoryInstanceList = Category.createCriteria().list(params) {
            if (params.search) {
                ilike("name", "%${params.search}%")
            }
        }
        [categoryInstanceList: categoryInstanceList]
    }

    def search = {
        redirect(action: "list", params: [offset: 0, search: params.q])
    }

    def create = {
        def categoryInstance = new Category()
        categoryInstance.properties = params
        return [categoryInstance: categoryInstance]
    }

    def save = {
        def categoryInstance = new Category(params)
        if (!categoryInstance.hasErrors() && categoryInstance.save()) {
            flash.message = "category.created"
            flash.args = [categoryInstance]
            flash.defaultMessage = "Category ${categoryInstance} created"
            redirect(action: "show", id: categoryInstance.id)
        }
        else {
            render(view: "create", model: [categoryInstance: categoryInstance])
        }
    }

    def show = {
        def categoryInstance = Category.get(params.id)
        if (!categoryInstance) {
            flash.message = "category.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "Category not found with id ${params.id}"
            redirect(action: "list")
        }
        else {
            return [categoryInstance: categoryInstance]
        }
    }

    def edit = {
        def categoryInstance = Category.get(params.id)
        if (!categoryInstance) {
            flash.message = "category.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "Category not found with id ${params.id}"
            redirect(action: "list")
        }
        else {
            return [categoryInstance: categoryInstance]
        }
    }

    def update = {
        def categoryInstance = Category.get(params.id)
        if (categoryInstance) {
            categoryInstance.properties = params
            if (!categoryInstance.hasErrors() && categoryInstance.save()) {
                flash.message = "category.updated"
                flash.args = [categoryInstance]
                flash.defaultMessage = "Category ${categoryInstance} updated"
                redirect(action: "show", id: categoryInstance.id)
            }
            else {
                render(view: "edit", model: [categoryInstance: categoryInstance])
            }
        }
        else {
            flash.message = "category.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "Category not found with id ${params.id}"
            redirect(action: "edit", id: params.id)
        }
    }

    def delete = {
        def categoryInstance = Category.get(params.id)
        if (categoryInstance) {
            def categoryInstanceToString = categoryInstance.toString()
            try {
                categoryInstance.delete()
                flash.message = "category.deleted"
                flash.args = [categoryInstanceToString]
                flash.defaultMessage = "Category ${categoryInstanceToString} deleted"
                redirect(action: "list")
            }
            catch (org.hibernate.exception.ConstraintViolationException e) {
                flash.message = "category.not.deleted"
                flash.args = [categoryInstanceToString]
                flash.defaultMessage = "Category ${categoryInstanceToString} not deleted"
                redirect(action: "show", id: params.id)
            }
        }
        else {
            flash.message = "category.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "Category not found with id ${params.id}"
            redirect(action: "list")
        }
    }
}
