/* Copyright 2009 Marcel Overdijk
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.grails.samples.sakila

class CityController extends BaseController {

    def index = { redirect(action: "list", params: params) }
    
    static allowedMethods = [save: "POST", update: "POST", delete: "POST", search: "POST"]

    def list = {
        if (params.max) params.max = Math.min(params.max.toInteger(), 100)
        restoreState([max: 10, offset: 0, sort: "id", order: "asc", search: ""])
        def cityInstanceList = City.createCriteria().list(params) {
            if (params.search) {
                or {
                    ilike("city", "%${params.search}%")
                    country {
                        ilike("country", "%${params.search}%")
                    }
                }
            }
        }
        [cityInstanceList: cityInstanceList]
    }

    def search = {
        redirect(action: "list", params: [offset: 0, search: params.q])
    }

    def create = {
        def cityInstance = new City()
        cityInstance.properties = params
        return [cityInstance: cityInstance]
    }

    def save = {
        def cityInstance = new City(params)
        if (!cityInstance.hasErrors() && cityInstance.save()) {
            flash.message = "city.created"
            flash.args = [cityInstance]
            flash.defaultMessage = "City ${cityInstance} created"
            redirect(action: "show", id: cityInstance.id)
        }
        else {
            render(view: "create", model: [cityInstance: cityInstance])
        }
    }

    def show = {
        def cityInstance = City.get(params.id)
        if (!cityInstance) {
            flash.message = "city.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "City not found with id ${params.id}"
            redirect(action: "list")
        }
        else {
            return [cityInstance: cityInstance]
        }
    }

    def edit = {
        def cityInstance = City.get(params.id)
        if (!cityInstance) {
            flash.message = "city.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "City not found with id ${params.id}"
            redirect(action: "list")
        }
        else {
            return [cityInstance: cityInstance]
        }
    }

    def update = {
        def cityInstance = City.get(params.id)
        if (cityInstance) {
            cityInstance.properties = params
            if (!cityInstance.hasErrors() && cityInstance.save()) {
                flash.message = "city.updated"
                flash.args = [cityInstance]
                flash.defaultMessage = "City ${cityInstance} updated"
                redirect(action: "show", id: cityInstance.id)
            }
            else {
                render(view: "edit", model: [cityInstance: cityInstance])
            }
        }
        else {
            flash.message = "city.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "City not found with id ${params.id}"
            redirect(action: "edit", id: params.id)
        }
    }

    def delete = {
        def cityInstance = City.get(params.id)
        if (cityInstance) {
            def cityInstanceToString = cityInstance.toString()
            try {
                cityInstance.delete()
                flash.message = "city.deleted"
                flash.args = [cityInstanceToString]
                flash.defaultMessage = "City ${cityInstanceToString} deleted"
                redirect(action: "list")
            }
            catch (org.hibernate.exception.ConstraintViolationException e) {
                flash.message = "city.not.deleted"
                flash.args = [cityInstanceToString]
                flash.defaultMessage = "City ${cityInstanceToString} not deleted"
                redirect(action: "show", id: params.id)
            }
        }
        else {
            flash.message = "city.not.found"
            flash.args = [params.id]
            flash.defaultMessage = "City not found with id ${params.id}"
            redirect(action: "list")
        }
    }
}
